<?php

namespace Drupal\home_page_for_roles\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpFoundation\RedirectResponse;

class HomePageForRolesSubscriber implements EventSubscriberInterface {

  protected $currentUser;
  protected $configFactory;

  public function __construct(
    AccountProxyInterface $current_user,
    ConfigFactoryInterface $config_factory
  ) {
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
  }

  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['onKernelRequest', 28],
    ];
  }

  public function onKernelRequest(RequestEvent $event) {
    $request = $event->getRequest();
    
    // Only act on the homepage
    if ($request->getPathInfo() !== '/') {
      return;
    }

    $config = $this->configFactory->get('home_page_for_roles.settings');
    $redirect_path = NULL;

    if (!$this->currentUser->isAuthenticated()) {
      $redirect_path = $config->get('anonymous_homepage');
    }
    else {
      // Check role-specific homepages
      $role_homepages = $config->get('role_homepages') ?? [];
      foreach ($this->currentUser->getRoles() as $role) {
        if (!empty($role_homepages[$role])) {
          $redirect_path = $role_homepages[$role];
          break;
        }
      }
      
      // If no role-specific homepage, use default authenticated homepage
      if (!$redirect_path) {
        $redirect_path = $config->get('authenticated_homepage');
      }
    }

    // Perform redirect if a path is set
    if ($redirect_path && $redirect_path !== '/') {
      $event->setResponse(new RedirectResponse($redirect_path));
    }
  }
}